package cz.muni.fi.pa165.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.api.auth.Role;
import cz.muni.fi.pa165.api.auth.SingleResponse;
import cz.muni.fi.pa165.api.auth.UserBasicViewDto;
import cz.muni.fi.pa165.api.ErrorMessage;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.lang.NonNull;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Calendar;
import java.util.List;

@Component
public class JwtAuthFilter extends OncePerRequestFilter {

    @Value("${authentication-service.address}")
    private String address;

    @Value("${authentication-service.port}")
    private String port;

    private final ObjectMapper objectMapper = new ObjectMapper();

    @Override
    protected void doFilterInternal(
            @NonNull HttpServletRequest request,
            @NonNull HttpServletResponse response,
            @NonNull FilterChain filterChain
    ) throws ServletException, IOException {
        final String authHeader = request.getHeader("Authorization");
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        if (authHeader == null || !authHeader.startsWith("Bearer")) {
            filterChain.doFilter(request, response);
            return;
        }
        String authUrl = "http://" + address +":" + port + "/api/users/authenticate";
        String requestPath = request.getServletPath();
        final String jwt = authHeader.substring(7);
        headers.setBearerAuth(jwt);

        HttpEntity<String> entity = new HttpEntity<>(null, headers);
        try {
            ResponseEntity<SingleResponse<UserBasicViewDto>> responseEntity = restTemplate.exchange(authUrl, HttpMethod.GET, entity, new ParameterizedTypeReference<SingleResponse<UserBasicViewDto>>() {});
            if (responseEntity.getStatusCode() == HttpStatus.FORBIDDEN || responseEntity.getStatusCode() == HttpStatus.UNAUTHORIZED) {
                setErrorResponse("Invalid authentication.", requestPath, HttpStatus.UNAUTHORIZED, response);
                return;
            }
            if (responseEntity.getBody().item().role() != Role.ADMIN) {
                setErrorResponse("User is not admin", requestPath, HttpStatus.UNAUTHORIZED, response);
                return;
            }

            UsernamePasswordAuthenticationToken authToken = new UsernamePasswordAuthenticationToken(
                    null, null, List.of(new SimpleGrantedAuthority(responseEntity.getBody().item().role().name()))
            );
            authToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
            SecurityContextHolder.getContext().setAuthentication(authToken);
        } catch (ResourceAccessException e) {
            setErrorResponse("Auth service is unavailable", requestPath, HttpStatus.INTERNAL_SERVER_ERROR, response);
            return;
        }
        catch (HttpClientErrorException e) {
            if(e.getStatusCode() == HttpStatus.FORBIDDEN || e.getStatusCode() == HttpStatus.UNAUTHORIZED) {
                setErrorResponse("Invalid authentication.", requestPath, HttpStatus.UNAUTHORIZED, response);
                return;
            }
            throw e;

        }
        catch (HttpServerErrorException e) {
            setErrorResponse("Remote server error occurred.", requestPath, HttpStatus.INTERNAL_SERVER_ERROR, response);
            return;
        }
        catch (Exception e) {
            setErrorResponse("Unexpected error occurred", requestPath, HttpStatus.INTERNAL_SERVER_ERROR, response);
            return;
        }
        filterChain.doFilter(request, response);
    }

    private void setErrorResponse(
            @NonNull String reason,
            @NonNull String path,
            @NonNull HttpStatus status,
            @NonNull HttpServletResponse response
    ) throws IOException {
        response.setStatus(status.value());
        response.setContentType("application/json");
        response.getWriter().write(objectMapper.writeValueAsString(new ErrorMessage(
                Calendar.getInstance().getTime().toString(),
                status.value(),
                status.getReasonPhrase(),
                reason,
                path
        )));

    }
}
