package cz.muni.fi.pa165.rest;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.WireMock;
import cz.muni.fi.pa165.api.auth.AuthResponse;
import cz.muni.fi.pa165.api.auth.Role;
import cz.muni.fi.pa165.api.auth.UserBasicViewDto;
import cz.muni.fi.pa165.api.genre.CreateGenreRequestDto;
import cz.muni.fi.pa165.api.image.CreateImageRequestDto;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.web.servlet.MockMvc;

import java.io.IOException;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static com.github.tomakehurst.wiremock.client.WireMock.aResponse;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_CLASS)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class ImageControllerTest {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    ObjectMapper objectMapper;

    private static final WireMockServer wireMockServer = new WireMockServer(wireMockConfig().port(8088));

    @BeforeAll
    static void setupMockAuthServer() throws JsonProcessingException {
        try {
            ObjectMapper objectMapperStat = new ObjectMapper();
            wireMockServer.start();
            String authResponse = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserBasicViewDto(1, "admin", "admin@mockery.com", Role.ADMIN),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer adminJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponse)
                            )
            );
            String authResponseUser = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserBasicViewDto(1, "user", "admin@mockery.com", Role.USER),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer userJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponseUser)
                            )
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Order(1)
    @Test
    void getImages() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/api/images"))
              .andExpect(status().isOk())
              .andExpect(content().json(
                      """
                                 {
                                 	"content": [
                                 		{
                                 			"id": 1,
                                 			"url": "www.random.com/image1",
                                 			"description": "Nice title image"
                                 		},
                                 		{
                                 			"id": 2,
                                 			"url": "www.random.com/image2",
                                 			"description": "Nice image"
                                 		},
                                 		{
                                 			"id": 3,
                                 			"url": "www.random.com/image3",
                                 			"description": "Ugly title image"
                                 		},
                                 		{
                                 			"id": 4,
                                 			"url": "www.random.com/image4",
                                 			"description": "Low res image"
                                 		}
                                 	],
                                 	"message": "Success"
                                 }
                                 """
              ));
    }

    @Order(2)
    @Rollback(false)
    @Test
    void createImage() throws Exception {
        String json = objectMapper.writeValueAsString(new CreateImageRequestDto("https://i1.sndcdn.com/artworks-dVH0EE65EwUOKdvd-NfhGug-t500x500.jpg", "test"));
        mockMvc.perform(MockMvcRequestBuilders.post("/api/images")
                    .contentType("application/json")
                    .header("Authorization", "Bearer adminJWT")
                    .content(json)
                )
                .andExpect(status().isCreated())
                .andExpect(content().json("""
                        {
                            "content": {
                                 "id": 5,
                                 "url": "https://i1.sndcdn.com/artworks-dVH0EE65EwUOKdvd-NfhGug-t500x500.jpg",
                                 "description": "test"
                            },
                            "message":"Success"
                        }
                        """
                        )
                );
    }

    @Order(3)
    @Test
    void verifyImageCreated() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/api/images"))
                .andExpect(status().isOk())
                .andExpect(content().json(
                       """
                       {
                           "content": [
                               {
                                   "id": 1,
                                   "url": "www.random.com/image1",
                                   "description": "Nice title image"
                               },
                               {
                                   "id": 2,
                                   "url": "www.random.com/image2",
                                   "description": "Nice image"
                               },
                               {
                                   "id": 3,
                                   "url": "www.random.com/image3",
                                   "description": "Ugly title image"
                               },
                               {
                                   "id": 4,
                                   "url": "www.random.com/image4",
                                   "description": "Low res image"
                               },
                               {
                                 "id": 5,
                                 "url": "https://i1.sndcdn.com/artworks-dVH0EE65EwUOKdvd-NfhGug-t500x500.jpg",
                                 "description": "test"
                               }
                           ],
                           "message": "Success"
                       }
                       """
                ));
    }

    @Order(4)
    @Test
    void getImage() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/api/images/1"))
               .andExpect(status().isOk())
               .andExpect(content().json(
                        """
                        {
                            "content": {
                                 "id": 1,
                                 "url": "www.random.com/image1",
                                 "description": "Nice title image"
                            },
                            "message":"Success"
                        }
                        """
                ));
    }

    @Order(5)
    @Test
    void deleteImage() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.delete("/api/images/5")
                    .header("Authorization", "Bearer adminJWT")
                )
                .andExpect(status().isNoContent())
                .andExpect(content().json(
                        """
                        {
                            "message":"Success"
                        }
                        """
                ));
    }

    @Order(6)
    @Test
    void verifyImageDeleted() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/api/images"))
                .andExpect(status().isOk())
                .andExpect(content().json(
                        """
                                   {
                                       "content": [
                                           {
                                               "id": 1,
                                               "url": "www.random.com/image1",
                                               "description": "Nice title image"
                                           },
                                           {
                                               "id": 2,
                                               "url": "www.random.com/image2",
                                               "description": "Nice image"
                                           },
                                           {
                                               "id": 3,
                                               "url": "www.random.com/image3",
                                               "description": "Ugly title image"
                                           },
                                           {
                                               "id": 4,
                                               "url": "www.random.com/image4",
                                               "description": "Low res image"
                                           }
                                       ],
                                       "message": "Success"
                                   }
                                   """
                ));
    }

    @Order(7)
    @Test
    void authWorkingTest() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.delete("/api/images/5")
                        .header("Authorization", "Bearer userJWT")
                )
                .andExpect(status().isUnauthorized());
    }


    @AfterAll
    static void tearDownMockAuthServer() throws IOException {
        wireMockServer.stop();
    }
}
