package cz.muni.fi.pa165.rest;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.WireMock;
import cz.muni.fi.pa165.api.auth.AuthResponse;
import cz.muni.fi.pa165.api.auth.Role;
import cz.muni.fi.pa165.api.auth.UserBasicViewDto;
import cz.muni.fi.pa165.api.genre.CreateGenreRequestDto;
import cz.muni.fi.pa165.api.movie.CreateMovieRequestDto;
import cz.muni.fi.pa165.api.movieStakeholder.CreateMovieStakeHolderRequestDto;
import org.junit.jupiter.api.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.test.web.servlet.result.StatusResultMatchers;

import java.io.IOException;
import java.util.List;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static com.github.tomakehurst.wiremock.client.WireMock.aResponse;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_CLASS)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class MovieControllerTest {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    ObjectMapper objectMapper;

    private static final WireMockServer wireMockServer = new WireMockServer(wireMockConfig().port(8088));

    @BeforeAll
    static void setupMockAuthServer() throws JsonProcessingException {
        try {
            ObjectMapper objectMapperStat = new ObjectMapper();
            wireMockServer.start();
            String authResponse = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserBasicViewDto(1, "admin", "admin@mockery.com", Role.ADMIN),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer adminJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponse)
                            )
            );
            String authResponseUser = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserBasicViewDto(1, "user", "admin@mockery.com", Role.USER),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer userJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponseUser)
                            )
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Order(1)
    @Test
    void getMoviesTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		}
                                	],
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(2)
    @Rollback(false)
    @Test
    void postMovieTest() throws Exception {
        String json = objectMapper.writeValueAsString(new CreateMovieRequestDto(
                "Aquaman",
                2019,
                "Cringe movie",
                4L,
                new Long[]{1L},
                new Long[]{4L, 5L},
                new Long[]{3L, 4L}
        ));
        this.mockMvc.perform(MockMvcRequestBuilders.post("/api/movies")
                        .contentType(MediaType.APPLICATION_JSON)
                        .header("Authorization", "Bearer adminJWT")
                        .content(json)
                )
                .andExpect(status().isCreated())
                .andExpect(content().json(
                        """
                                {
                                	"item": {
                                		"id": 2,
                                		"title": "Aquaman",
                                		"year": 2019,
                                		"director": {
                                			"id": 4,
                                			"name": "Jason",
                                			"surname": "Momoa"
                                		},
                                		"genres": [
                                			{
                                				"id": 1,
                                				"name": "Drama"
                                			}
                                		],
                                		"actors": [
                                			{
                                				"id": 4,
                                				"name": "Jason",
                                				"surname": "Momoa"
                                			},
                                			{
                                				"id": 5,
                                				"name": "Amber",
                                				"surname": "Heard"
                                			}
                                		],
                                		"images": [
                                			{
                                				"id": 3,
                                				"imageUrl": "www.random.com/image3",
                                				"description": "Ugly title image"
                                			},
                                			{
                                				"id": 4,
                                				"imageUrl": "www.random.com/image4",
                                				"description": "Low res image"
                                			}
                                		]
                                	},
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(3)
    @Test
    void verifyPostMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		},
                                		{
                                			"id": 2,
                                			"title": "Aquaman",
                                			"year": 2019,
                                			"director": {
                                				"id": 4,
                                				"name": "Jason",
                                				"surname": "Momoa"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 4,
                                					"name": "Jason",
                                					"surname": "Momoa"
                                				},
                                				{
                                					"id": 5,
                                					"name": "Amber",
                                					"surname": "Heard"
                                				}
                                			],
                                			"images": [
                                                {
                                                    "id": 3,
                                                    "imageUrl": "www.random.com/image3",
                                                    "description": "Ugly title image"
                                                },
                                                {
                                                    "id": 4,
                                                    "imageUrl": "www.random.com/image4",
                                                    "description": "Low res image"
                                                }
                                		    ]
                                		}
                                	],
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(4)
    @Test
    void getMoviesPagedTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies/paged")
                        .queryParam("page", "0")
                        .queryParam("size", "1"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		}
                                	],
                                	"total": 2,
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(5)
    @Test
    void getSingleMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies/1"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"item": {
                                		"id": 1,
                                		"title": "The Godfather",
                                		"year": 2009,
                                		"director": {
                                			"id": 1,
                                			"name": "Francis",
                                			"surname": "Coppola"
                                		},
                                		"genres": [
                                			{
                                				"id": 1,
                                				"name": "Drama"
                                			}
                                		],
                                		"actors": [
                                			{
                                				"id": 2,
                                				"name": "Al",
                                				"surname": "Pacino"
                                			}
                                		],
                                		"images": []
                                	},
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(6)
    @Rollback(false)
    @Test
    void updateMovieTest() throws Exception {
        String json = objectMapper.writeValueAsString(new CreateMovieRequestDto(
                "Aquaman",
                2019,
                "Cringe movie",
                4L,
                new Long[]{1L},
                new Long[]{4L},
                new Long[]{3L}
        ));
        this.mockMvc.perform(MockMvcRequestBuilders.put("/api/movies/2")
                        .contentType(MediaType.APPLICATION_JSON)
                        .header("Authorization", "Bearer adminJWT")
                        .content(json))
                .andExpect(status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"item": {
                                		"id": 2,
                                		"title": "Aquaman",
                                		"year": 2019,
                                		"director": {
                                			"id": 4,
                                			"name": "Jason",
                                			"surname": "Momoa"
                                		},
                                		"genres": [
                                			{
                                				"id": 1,
                                				"name": "Drama"
                                			}
                                		],
                                		"actors": [
                                			{
                                				"id": 4,
                                				"name": "Jason",
                                				"surname": "Momoa"
                                			}
                                		],
                                		"images": [
                                			{
                                				"id": 3,
                                				"imageUrl": "www.random.com/image3",
                                				"description": "Ugly title image"
                                			}
                                		]
                                	},
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(7)
    @Test
    void verifyUpdateMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		},
                                		{
                                  			"id": 2,
                                  			"title": "Aquaman",
                                  			"year": 2019,
                                  			"director": {
                                  				"id": 4,
                                  				"name": "Jason",
                                  				"surname": "Momoa"
                                  			},
                                  			"genres": [
                                  				{
                                  					"id": 1,
                                  					"name": "Drama"
                                  				}
                                  			],
                                  			"actors": [
                                  				{
                                  					"id": 4,
                                  					"name": "Jason",
                                  					"surname": "Momoa"
                                  				}
                                  			],
                                  			"images": [
                                  				{
                                  					"id": 3,
                                  					"imageUrl": "www.random.com/image3",
                                  					"description": "Ugly title image"
                                  				},
                                  				{
                                  					"id": 4,
                                  					"imageUrl": "www.random.com/image4",
                                  					"description": "Low res image"
                                  				}
                                  			]
                                  		}
                                	],
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(8)
    @Rollback(false)
    @Test
    void deleteMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.delete("/api/movies/2")
                        .header("Authorization", "Bearer adminJWT")
                )
                .andExpect(MockMvcResultMatchers.status().isNoContent())
                .andExpect(content().json(
                        """
                                {
                                    "message": "Success"
                                }
                                """
                ));
    }

    @Order(9)
    @Rollback(false)
    @Test
    void verifyDeleteMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		}
                                	],
                                	"message": "Success"
                                }
                                """
                ));
    }

    @Order(10)
    @Rollback(false)
    @Test
    void getRandomMovieTest() throws Exception {
        this.mockMvc.perform(MockMvcRequestBuilders.get("/api/movies/random"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(content().json(
                        """
                                {
                                	"content": [
                                		{
                                			"id": 1,
                                			"title": "The Godfather",
                                			"year": 2009,
                                			"director": {
                                				"id": 1,
                                				"name": "Francis",
                                				"surname": "Coppola"
                                			},
                                			"genres": [
                                				{
                                					"id": 1,
                                					"name": "Drama"
                                				}
                                			],
                                			"actors": [
                                				{
                                					"id": 2,
                                					"name": "Al",
                                					"surname": "Pacino"
                                				}
                                			],
                                			"images": []
                                		}
                                	],
                                	"message": "Success"
                                }
                                """
                ));
    }

    @AfterAll
    static void tearDownMockAuthServer() throws IOException {
        wireMockServer.stop();
    }
}
