package cz.muni.fi.pa165.config;

import cz.muni.fi.pa165.data.api.user.SingleResponse;
import cz.muni.fi.pa165.data.api.user.UserViewDto;
import cz.muni.fi.pa165.data.model.auth.Role;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.*;
import org.springframework.lang.NonNull;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.List;

@Component
public class JwtAuthFilter extends OncePerRequestFilter {
    @Value("${services.user.url}")
    private String address;

    @Value("${services.user.port}")
    private String port;

    @Value("${services.user.path}")
    private String authEndpoint;

    @Override
    protected void doFilterInternal(
            @NonNull HttpServletRequest request,
            @NonNull HttpServletResponse response,
            @NonNull FilterChain filterChain
    ) throws ServletException, IOException {
        final String authHeader = request.getHeader("Authorization");
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        if (authHeader == null || !authHeader.startsWith("Bearer")) {
            filterChain.doFilter(request, response);
            return;
        }
        String authUrl = address + ":" + port +  authEndpoint + "/authenticate";
        final String jwt = authHeader.substring(7);
        headers.setBearerAuth(jwt);

        HttpEntity<String> entity = new HttpEntity<>(null, headers);
        try {
            ResponseEntity<SingleResponse<UserViewDto>> responseEntity = restTemplate.exchange(authUrl, HttpMethod.GET, entity, new ParameterizedTypeReference<>() {});
            if (responseEntity.getBody().getItem().getRole() != Role.ADMIN) {
                setErrorResponse("User is not admin", HttpStatus.UNAUTHORIZED, response);
                return;
            }

            UsernamePasswordAuthenticationToken authToken = new UsernamePasswordAuthenticationToken(
                    null, null, List.of(new SimpleGrantedAuthority(responseEntity.getBody().getItem().getRole().name()))
            );
            authToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
            SecurityContextHolder.getContext().setAuthentication(authToken);
        } catch (ResourceAccessException e) {
            setErrorResponse("Auth service is unavailable", HttpStatus.INTERNAL_SERVER_ERROR, response);
            return;
        }
        catch (HttpClientErrorException e) {
            if(e.getStatusCode() == HttpStatus.FORBIDDEN) {
                setErrorResponse("Invalid authentication.", HttpStatus.UNAUTHORIZED, response);
                return;
            }
            throw e;

        }
        catch (Exception e) {
            setErrorResponse("Unknown error.", HttpStatus.INTERNAL_SERVER_ERROR, response);
            return;
        }
        filterChain.doFilter(request, response);
    }

    private void setErrorResponse(
            @NonNull String reason,
            @NonNull HttpStatus status,
            @NonNull HttpServletResponse response
    ) throws IOException {
        response.setStatus(status.value());
        response.setContentType("application/json");
        response.getWriter().write(reason);
    }
}
