package cz.muni.fi.pa165.rest;

import static com.github.tomakehurst.wiremock.client.WireMock.aResponse;
import static com.github.tomakehurst.wiremock.client.WireMock.containing;
import static com.github.tomakehurst.wiremock.client.WireMock.urlEqualTo;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.tomakehurst.wiremock.WireMockServer;
import com.github.tomakehurst.wiremock.client.WireMock;
import cz.muni.fi.pa165.data.api.user.AuthResponse;
import cz.muni.fi.pa165.data.api.user.UserViewDto;
import cz.muni.fi.pa165.data.model.auth.Role;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
class CriterionControllerTest {

    private static final WireMockServer wireMockServer = new WireMockServer(wireMockConfig().port(8088));

    @Autowired
    private MockMvc mockMvc;

    @BeforeAll
    static void setupMockAuthServer() {
        try {
            ObjectMapper objectMapperStat = new ObjectMapper();
            wireMockServer.start();
            String authResponse = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserViewDto(1, "admin", "admin@mockery.com", Role.ADMIN),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer adminJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponse)
                            )
            );
            String authResponseUser = objectMapperStat.writeValueAsString(new AuthResponse(
                    new UserViewDto(1, "user", "admin@mockery.com", Role.USER),
                    "Successfully authenticated"
            ));
            wireMockServer.stubFor(
                    WireMock.get(urlEqualTo("/api/users/authenticate"))
                            .withHeader("Authorization", containing("Bearer userJWT"))
                            .willReturn(aResponse()
                                    .withHeader("Content-Type", "application/json")
                                    .withBody(authResponseUser)
                            )
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @AfterAll
    static void tearDownMockAuthServer() {
        wireMockServer.stop();
    }

    @Test
    void getAllTest() throws Exception {
        this.mockMvc.perform(get("/api/criterion"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        [
                        	{
                        		"id": 1,
                        		"name": "Criterion 1",
                        		"description": "Description 1"
                        	},
                        	{
                        		"id": 2,
                        		"name": "Criterion 2",
                        		"description": "Description 2"
                        	},
                        	{
                        		"id": 3,
                        		"name": "Criterion 3",
                        		"description": "Description 3"
                        	}
                        ]
                        """));
    }

    @Test
    void getOneTest() throws Exception {
        this.mockMvc.perform(get("/api/criterion/1"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                         	"id": 1,
                         	"name": "Criterion 1",
                         	"description": "Description 1"
                         }
                        """));
    }

    @Test
    void addTest() throws Exception {
        this.mockMvc.perform(post("/api/criterion")
                        .header("Authorization", "Bearer adminJWT")
                        .content("""
                        {
                          "name": "Acting",
                          "description": "Acting of the actors"
                        }
                        """)
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                            "id": 4,
                            "name": "Acting",
                            "description": "Acting of the actors"
                        }
                        """));
    }

    @Test
    void deleteTest() throws Exception {
        this.mockMvc.perform(
                delete("/api/criterion/1")
                        .header("Authorization", "Bearer adminJWT")
                )
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                        	"id": 1,
                        	"name": "Criterion 1",
                        	"description": "Description 1"
                        }
                        """));
    }

    @Test
    void editTest() throws Exception {
        this.mockMvc.perform(
                patch("/api/criterion")
                        .header("Authorization", "Bearer adminJWT")
                        .content("""
                        {
                          "id": 1,
                          "name": "Sound effects",
                          "description": "Various sound effects"
                        }
                        """)
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json("""
                        {
                          "id": 1,
                          "name": "Sound effects",
                          "description": "Various sound effects"
                        }
                        """));
    }
}
