package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.model.Criterion;
import cz.muni.fi.pa165.data.model.Movie;
import cz.muni.fi.pa165.data.model.Rating;
import cz.muni.fi.pa165.data.model.User;
import cz.muni.fi.pa165.data.repository.CriterionRepository;
import cz.muni.fi.pa165.data.repository.MovieRepository;
import cz.muni.fi.pa165.data.repository.RatingRepository;
import cz.muni.fi.pa165.data.repository.UserRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.BeforeEach;
import org.mockito.Mockito;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.Mockito.when;

public class RatingServiceTest {

    private RatingService ratingService;

    private RatingRepository ratingRepository;
    private MovieRepository movieRepository;
    private UserRepository userRepository;
    private CriterionRepository criterionRepository;

    @BeforeEach
    public void setup() {
        ratingRepository = Mockito.mock(RatingRepository.class);
        movieRepository = Mockito.mock(MovieRepository.class);
        userRepository = Mockito.mock(UserRepository.class);
        criterionRepository = Mockito.mock(CriterionRepository.class);

        ratingService = new RatingService(ratingRepository, criterionRepository, movieRepository, userRepository);
    }

    @Test
    public void getAllByCriteriaTest() {
        when(ratingRepository.findAllByUserIdAndMovieIdAndCriterionId(any(), any(), any(), any()))
                .thenReturn(new PageImpl<>(List.of(getRating(10, -1, -1, -1))));

        var found = ratingService.getAllByCriteria(Optional.of(-1), Optional.of(-1), Optional.of(-1), Pageable.unpaged());

        assertEquals(1, found.getTotalElements());
        assertEquals(-1, found.getContent().get(0).getCriterion().getId());
        assertEquals(-1, found.getContent().get(0).getMovie().getId());
        assertEquals(-1, found.getContent().get(0).getUser().getId());
        assertEquals(10, found.getContent().get(0).getValue());
    }

    @Test
    public void createTest() {
        when(userRepository.findById(any())).thenReturn(Optional.of(new User() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(movieRepository.findById(any())).thenReturn(Optional.of(new Movie() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(criterionRepository.findById(any())).thenReturn(Optional.of(new Criterion() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(ratingRepository.findByUserIdAndMovieIdAndCriterionId(anyInt(), anyInt(), anyInt()))
                .thenReturn(Optional.empty());
        when(ratingRepository.save(any()))
                .thenReturn(getRating(10, -1, -1, -1));

        var rating = ratingService.upsert(getRating(10, -1, -1, -1));

        assertEquals(-1, rating.getCriterion().getId());
        assertEquals(-1, rating.getMovie().getId());
        assertEquals(-1, rating.getUser().getId());
        assertEquals(10, rating.getValue());
    }

    @Test
    public void updateTest() {
        when(userRepository.findById(any())).thenReturn(Optional.of(new User() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(movieRepository.findById(any())).thenReturn(Optional.of(new Movie() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(criterionRepository.findById(any())).thenReturn(Optional.of(new Criterion() {
            @Override
            public int getId() {
                return -1;
            }
        }));

        when(ratingRepository.findByUserIdAndMovieIdAndCriterionId(anyInt(), anyInt(), anyInt()))
                .thenReturn(Optional.of(getRating(5, -1, -1, -1)));
        when(ratingRepository.save(any()))
                .thenReturn(getRating(10, -1, -1, -1));

        var rating = ratingService.upsert(getRating(10, -1, -1, -1));

        assertEquals(-1, rating.getCriterion().getId());
        assertEquals(-1, rating.getMovie().getId());
        assertEquals(-1, rating.getUser().getId());
        assertEquals(10, rating.getValue());
    }

    @Test
    public void deleteTest() {
        when(ratingRepository.findByUserIdAndMovieIdAndCriterionId(anyInt(), anyInt(), anyInt()))
                .thenReturn(Optional.of(getRating(10, -1, -1, -1)));

        var rating = ratingService.delete(-1, -1, -1);

        assertEquals(-1, rating.getCriterion().getId());
        assertEquals(-1, rating.getMovie().getId());
        assertEquals(-1, rating.getUser().getId());
        assertEquals(10, rating.getValue());
    }

    private Rating getRating(int value, int criterionId, int movieId, int userId) {
        var criterionMock = new Criterion();
        criterionMock.setId(criterionId);

        var movieMock = new Movie();
        movieMock.setId(movieId);

        var userMock = new User();
        userMock.setId(userId);

        return new Rating(value, criterionMock, movieMock, userMock);
    }
}