package cz.muni.fi.pa165.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.api.common.ErrorMessage;
import cz.muni.fi.pa165.exceptions.JwtFailException;
import cz.muni.fi.pa165.service.JwtService;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.lang.NonNull;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Component
@RequiredArgsConstructor
public class JwtAuthFilter extends OncePerRequestFilter {

    private final JwtService jwtService;
    private final UserDetailsService userDetailsService;
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final ArrayList<String> skipUrls = new ArrayList<>(List.of("/api/auth/register", "/api/auth/login"));

    @Override
    protected void doFilterInternal(
            @NonNull HttpServletRequest request,
            @NonNull HttpServletResponse response,
            @NonNull FilterChain filterChain
    ) throws ServletException, IOException {
        final String authHeader = request.getHeader("Authorization");

        if (authHeader == null || !authHeader.startsWith("Bearer")) {
            setErrorResponse(HttpStatus.BAD_REQUEST, "Missing valid Bearer token", response);
            return;
        }

        try {
            final String jwt = authHeader.substring(7);
            final String email = jwtService.getUserName(jwt);
            if (email != null && SecurityContextHolder.getContext().getAuthentication() == null) {
                UserDetails userDetails = this.userDetailsService.loadUserByUsername(email);
                if (jwtService.isTokenValid(jwt, userDetails)) {
                    var authToken = new UsernamePasswordAuthenticationToken(
                            userDetails, null, userDetails.getAuthorities()
                    );
                    authToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
                    SecurityContextHolder.getContext().setAuthentication(authToken);
                }
            }
        } catch (JwtFailException e) {
            setErrorResponse(e.getHttpStatus(), e.getMessage(), response);
            return;
        } catch (Exception e) {
            setErrorResponse(HttpStatus.INTERNAL_SERVER_ERROR, e.getMessage(), response);
            return;
        }

        filterChain.doFilter(request, response);
    }

    private void setErrorResponse(
            @NonNull HttpStatus status,
            @NonNull String message,
            @NonNull HttpServletResponse response
    ) throws IOException {
        response.setStatus(status.value());
        response.setContentType("application/json");
        var errorMsg = new ErrorMessage(status.getReasonPhrase(), message);
        response.getWriter().write(objectMapper.writeValueAsString(errorMsg));
    }

    @Override
    protected boolean shouldNotFilter(@NonNull HttpServletRequest request) {
        if(!(new AntPathRequestMatcher("/api/**").matches(request))) {
            return true;
        }
        return skipUrls.stream().anyMatch(url -> new AntPathRequestMatcher(url).matches(request));
    }
}
