package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.api.auth.login.LoginRequestDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterRequestDto;
import cz.muni.fi.pa165.api.auth.TokenDto;
import cz.muni.fi.pa165.data.model.Role;
import cz.muni.fi.pa165.data.model.User;
import cz.muni.fi.pa165.data.repository.UserRepository;
import cz.muni.fi.pa165.exceptions.ApiException;
import lombok.RequiredArgsConstructor;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionSystemException;

@Service
@RequiredArgsConstructor
public class AuthService {

    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final JwtService jwtService;
    private final AuthenticationManager authenticationManager;

    public TokenDto authenticate(LoginRequestDto req) {
        try {
            var authToken = new UsernamePasswordAuthenticationToken(req.getEmail(), req.getPassword());
            authenticationManager.authenticate(authToken);

            var user = userRepository.findByEmail(req.getEmail()).orElseThrow();
            String jwt = jwtService.generateJwt(user);
            return TokenDto.builder().token(jwt).build();
        } catch (Exception e) {
            throw new ApiException(HttpStatus.UNAUTHORIZED, "Invalid email or password");
        }
    }

    public TokenDto register(RegisterRequestDto req) {
        User user = User.builder()
                .name(req.getName())
                .email(req.getEmail())
                .role(Role.valueOf(req.getRole()))
                .password(passwordEncoder.encode(req.getPassword()))
                .build();

        // The validation will be moved to upper layers to get more precise error messages
        try {
            userRepository.save(user);
        } catch (DataIntegrityViolationException e) {
            throw new ApiException(HttpStatus.CONFLICT, "User with this email already exists");
        } catch (TransactionSystemException e) {
            throw new ApiException(HttpStatus.BAD_REQUEST, "User is not valid");
        } catch (Exception e) {
            throw new ApiException(HttpStatus.INTERNAL_SERVER_ERROR, e.getMessage());
        }

        String jwt = jwtService.generateJwt(user);
        return TokenDto.builder().token(jwt).build();
    }

}
