package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.exceptions.JwtFailException;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.io.Decoders;
import io.jsonwebtoken.security.Keys;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Value;

import java.security.Key;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

@Service
public class JwtService {

    @Value("${app.jwt.secret}")
    private String secretKey;

    @Value("${app.jwt.expiration}")
    private long expiration;

    public String getUserName(String jwt) throws JwtFailException {
        return extractClaims(jwt, Claims::getSubject);
    }

    public <T> T extractClaims(String jwt, Function<Claims, T> claimsResolver) throws JwtFailException {
        final Claims claims = extractAllClaims(jwt);
        return claimsResolver.apply(claims);
    }

    private Claims extractAllClaims(String jwt) throws JwtFailException {
        try {
            return Jwts.parserBuilder()
                    .setSigningKey(getSignSecretKey())
                    .build()
                    .parseClaimsJws(jwt)
                    .getBody();
        } catch (Exception e) {
            throw new JwtFailException(HttpStatus.UNAUTHORIZED, "Bearer token is not valid");
        }
    }

    private Key getSignSecretKey() {
        byte[] keyBytes = Decoders.BASE64.decode(secretKey);
        return Keys.hmacShaKeyFor(keyBytes);
    }

    public boolean isTokenValid(String jwt, UserDetails userDetails) throws JwtFailException {
        final String userName = getUserName(jwt);
        return (userName.equals(userDetails.getUsername())) && !isJwtExpired(jwt);
    }

    public boolean isJwtExpired(String jwt) throws JwtFailException {
        return extractClaims(jwt, Claims::getExpiration).before(new Date());
    }

    public String generateJwt(Map<String, Object> claims, UserDetails userDetails) {
        return Jwts.builder()
                .setClaims(claims)
                .setSubject(userDetails.getUsername())
                .setIssuedAt(new Date(System.currentTimeMillis()))
                .setExpiration(new Date(System.currentTimeMillis() + expiration))
                .signWith(getSignSecretKey(), SignatureAlgorithm.HS256)
                .compact();
    }


    public String generateJwt(UserDetails userDetails) {
        return generateJwt(new HashMap<>(), userDetails);
    }

}
