package integration.cz.muni.fi.pa168;


import cz.muni.fi.pa165.App;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(classes = App.class)
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
public class AuthTest {

    @Autowired
    private MockMvc mockMvc;

    @Test
    void successfulRegistrationTest() throws Exception {
        String bodyContent = """
            {
                "name": "John Watson",
                "email": "john.watson@doctor.com",
                "password": "WarIsOverNow",
                "role": "USER"
            }
        """;

        String expectedMessage = "User john.watson@doctor.com is registered";

        var request = post("/api/auth/register")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.message").value(expectedMessage))
                .andExpect(jsonPath("$.item.token").exists());;
    }

    @Test
    void userAlreadyExistsRegistrationTest() throws Exception {
        String bodyContent = """
            {
                "name": "Sherlock Holmes",
                "email": "sherlock.holmes@detective.com",
                "password": "WarIsOverNow",
                "role": "USER"
            }
        """;

        String expectedMessage = "User with this email already exists";

        var request = post("/api/auth/register")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().is4xxClientError())
                .andExpect(jsonPath("$.description").value(expectedMessage));
    }

    @Test
    void invalidUserRegistrationTest() throws Exception {
        String bodyContent = """
            {
                "email": "sherlock.holmes@detective.com",
                "password": "WarIsOverNow",
                "role": "USER"
            }
        """;

        String expectedMessage = "User is not valid";

        var request = post("/api/auth/register")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().is4xxClientError())
                .andExpect(jsonPath("$.description").value(expectedMessage));
    }

    @Test
    void loginTest() throws Exception {
        String bodyContent = """
            {
                "email": "sherlock.holmes@detective.com",
                "password": "HerlockSholmes"
            }
        """;

        String expectedMessage = "User sherlock.holmes@detective.com is logged in successfully";

        var request = post("/api/auth/login")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.message").value(expectedMessage))
                .andExpect(jsonPath("$.item.token").exists());
    }

    @Test
    void invalidCredentialsLoginTest() throws Exception {
        String bodyContent = """
            {
                "email": "holmes@detective.com",
                "password": "HerlockSholmes"
            }
        """;

        String expectedMessage = "Invalid email or password";

        var request = post("/api/auth/login")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isUnauthorized())
                .andExpect(jsonPath("$.description").value(expectedMessage));
    }

}
