package integration.cz.muni.fi.pa168;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.App;
import cz.muni.fi.pa165.api.auth.login.LoginResponseDto;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(classes = App.class)
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
public class UserTest {

    @Autowired
    private MockMvc mockMvc;
    @Autowired
    private ObjectMapper objectMapper;

    @Test
    void getUsersTest() throws Exception {

        var token = getLoginToken();

        String expectedContent = """
            {
                "item": [
                    {
                        "id": 1,
                        "name": "Sherlock Holmes",
                        "email": "sherlock.holmes@detective.com",
                        "role": "ADMIN"
                    },
                    {
                        "id": 2,
                        "name": "Jim Moriarty",
                        "email": "jim.moriarty@evil.com",
                        "role": "USER"
                    }
                ],
                "message": "Listed 2 users"
            }
        """;

        var request = get("/api/users")
                .header("Authorization", "Bearer " + token);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json(expectedContent));
    }

    @Test
    void missingTokenTest() throws Exception {

        String expectedContent = "Missing valid Bearer token";

        var request = get("/api/users");

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isBadRequest())
                .andExpect(jsonPath("$.description").value(expectedContent));
    }

    @Test
    void invalidTokenTest() throws Exception {

        String expectedContent = "Bearer token is not valid";

        var request = get("/api/users")
                .header("Authorization", "Bearer jn32j542");

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isUnauthorized())
                .andExpect(jsonPath("$.description").value(expectedContent));
    }


    @Test
    void authUserByTokenTest() throws Exception {

        var token = getLoginToken();

        String expectedContent = """
            {
                "item": {
                    "id": 1,
                    "name": "Sherlock Holmes",
                    "email": "sherlock.holmes@detective.com",
                    "role": "ADMIN"
                },
                "message": "User sherlock.holmes@detective.com is authenticated"
            }
        """;

        var request = get("/api/users/authenticate")
                .header("Authorization", "Bearer " + token);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json(expectedContent));
    }


    private String getLoginToken() throws Exception {
        var loginRequest = post("/api/auth/login")
                .content("{ \"email\": \"sherlock.holmes@detective.com\", \"password\": \"HerlockSholmes\" }")
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        var resp = this.mockMvc.perform(loginRequest)
                .andExpect(status().isOk())
                .andReturn()
                .getResponse()
                .getContentAsString();

        LoginResponseDto response = objectMapper.readValue(resp, LoginResponseDto.class);
        return response.getItem().getToken();
    }


}
