package unit.cz.muni.fi.pa168.rest;

import cz.muni.fi.pa165.App;
import cz.muni.fi.pa165.api.auth.TokenDto;
import cz.muni.fi.pa165.api.auth.login.LoginRequestDto;
import cz.muni.fi.pa165.api.auth.login.LoginResponseDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterRequestDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterResponseDto;
import cz.muni.fi.pa165.exceptions.ApiException;
import cz.muni.fi.pa165.facade.AuthFacade;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(classes = App.class)
@AutoConfigureMockMvc
public class AuthRestTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private AuthFacade authFacade;

    @Test
    void successfulRegistrationTest() throws Exception {
        String expectedMessage = "User john@mail.com is registered hh";
        String expectedToken = "expected token";

        String bodyContent = """
            {
                "name": "John",
                "email": "john@mail.com",
                "password": "Pass1",
                "role": "USER"
            }
        """;

        var reqDto = new RegisterRequestDto("John", "john@mail.com", "Pass1", "USER");
        var tokenDto = new TokenDto(expectedToken);
        var resDto = new RegisterResponseDto(tokenDto, expectedMessage);

        Mockito.when(authFacade.register(reqDto)).thenReturn(resDto);

        var request = post("/api/auth/register")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.message").value(expectedMessage))
                .andExpect(jsonPath("$.item.token").value(expectedToken));
    }

    @Test
    void existingUserRegistrationTest() throws Exception {
        String expectedDescription = "User with this email already exists";
        var conflictException = new ApiException(HttpStatus.CONFLICT, expectedDescription);

        String bodyContent = """
            {
                "name": "John",
                "email": "john@mail.com",
                "password": "Pass1",
                "role": "USER"
            }
        """;

        var reqDto = new RegisterRequestDto("John", "john@mail.com", "Pass1", "USER");

        Mockito.when(authFacade.register(reqDto)).thenThrow(conflictException);

        var request = post("/api/auth/register")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().is4xxClientError())
                .andExpect(jsonPath("$.description").value(expectedDescription));
    }

    @Test
    void loginTest() throws Exception {
        String expectedMessage = "User john@mail.com is logged in successfully";
        String expectedToken = "expectedToken";

        String bodyContent = """
            {
                "email": "john@mail.com",
                "password": "Pass1"
            }
        """;

        var reqDto = new LoginRequestDto("john@mail.com", "Pass1");
        var tokenDto = new TokenDto(expectedToken);
        var resDto = new LoginResponseDto(tokenDto, expectedMessage);

        Mockito.when(authFacade.login(reqDto)).thenReturn(resDto);

        var request = post("/api/auth/login")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.message").value(expectedMessage))
                .andExpect(jsonPath("$.item.token").value(expectedToken));
    }

    @Test
    void invalidPasswordLoginTest() throws Exception {
        String expectedDescription = "Invalid email or password";
        var authException = new ApiException(HttpStatus.UNAUTHORIZED, expectedDescription);

        String bodyContent = """
            {
                "email": "john@mail.com",
                "password": "Pass1"
            }
        """;

        var reqDto = new LoginRequestDto("john@mail.com", "Pass1");

        Mockito.when(authFacade.login(reqDto)).thenThrow(authException);

        var request = post("/api/auth/login")
                .content(bodyContent)
                .contentType(MediaType.APPLICATION_JSON_VALUE);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isUnauthorized())
                .andExpect(jsonPath("$.description").value(expectedDescription));
    }
}
