package unit.cz.muni.fi.pa168.rest;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.App;
import cz.muni.fi.pa165.api.user.UserAuthResponseDto;
import cz.muni.fi.pa165.api.user.UserBasicViewDto;
import cz.muni.fi.pa165.api.user.UserListResponseDto;
import cz.muni.fi.pa165.data.model.Role;
import cz.muni.fi.pa165.data.model.User;
import cz.muni.fi.pa165.data.repository.UserRepository;
import cz.muni.fi.pa165.exceptions.JwtFailException;
import cz.muni.fi.pa165.facade.UserFacade;
import cz.muni.fi.pa165.service.JwtService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.web.servlet.MockMvc;

import java.util.List;
import java.util.Optional;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest(classes = App.class)
@AutoConfigureMockMvc
public class UserRestTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private UserFacade userFacade;

    @MockBean
    private UserRepository userRepository;

    @MockBean
    private JwtService jwtService;

    @Autowired
    private ObjectMapper objectMapper;

    private final String fakeAuthToken = "someFakeAuthToken";

    @BeforeEach
    public void setupAuth() throws JwtFailException {
        var authUser = new User(1, "Auth User", "auth.user@mail.com", Role.USER, "Pass1");

        Mockito.when(jwtService.getUserName(fakeAuthToken)).thenReturn(authUser.getEmail());
        Mockito.when(userRepository.findByEmail(authUser.getEmail())).thenReturn(Optional.of(authUser));
        Mockito.when(jwtService.isTokenValid(fakeAuthToken, authUser)).thenReturn(true);
    }

    @Test
    void allUsersTest() throws Exception {
        var usersDto = List.of(
                new UserBasicViewDto(2, "Sherlock Holmes", "sherlock@mail.com", Role.ADMIN),
                new UserBasicViewDto(3, "Jim Moriarty", "jim@mail.com", Role.ADMIN)
        );

        var resDto = new UserListResponseDto(usersDto, "Listed 2 users");
        var expectedResponseBody = objectMapper.writeValueAsString(resDto);

        Mockito.when(userFacade.getAllUsers()).thenReturn(resDto);

        var request = get("/api/users").header("Authorization", "Bearer " + fakeAuthToken);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json(expectedResponseBody));
    }

    @Test
    void emptyUsersTest() throws Exception {
        List<UserBasicViewDto> usersDto = List.of();

        var resDto = new UserListResponseDto(usersDto, "Listed 0 users");
        var expectedResponseBody = objectMapper.writeValueAsString(resDto);

        Mockito.when(userFacade.getAllUsers()).thenReturn(resDto);

        var request = get("/api/users").header("Authorization", "Bearer " + fakeAuthToken);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json(expectedResponseBody));
    }

    @Test
    void unauthorizedUserListTest() throws Exception {
        String expectedContent = "Missing valid Bearer token";

        var request = get("/api/users");

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isBadRequest())
                .andExpect(jsonPath("$.description").value(expectedContent));
    }

    @Test
    void authenticateUserTest() throws Exception {
        var userDto = new UserBasicViewDto(1, "Sherlock Holmes", "sherlock@mail.com", Role.ADMIN);
        var resMessage = "User sherlock.holmes@detective.com is authenticated";

        var resDto = new UserAuthResponseDto(userDto, resMessage);
        var expectedContent = objectMapper.writeValueAsString(resDto);
        Mockito.when(userFacade.getUserByEmail(Mockito.anyString())).thenReturn(resDto);

        var request = get("/api/users/authenticate")
                .header("Authorization", "Bearer " + fakeAuthToken);

        this.mockMvc.perform(request)
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().json(expectedContent));
    }

}
