package unit.cz.muni.fi.pa168.service;

import cz.muni.fi.pa165.service.AuthService;
import cz.muni.fi.pa165.service.JwtService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import java.util.Optional;

import cz.muni.fi.pa165.api.auth.TokenDto;
import cz.muni.fi.pa165.api.auth.login.LoginRequestDto;
import cz.muni.fi.pa165.api.auth.registration.RegisterRequestDto;
import cz.muni.fi.pa165.data.model.Role;
import cz.muni.fi.pa165.data.model.User;
import cz.muni.fi.pa165.data.repository.UserRepository;
import cz.muni.fi.pa165.exceptions.ApiException;

public class AuthServiceTest {
    private AuthService authService;
    private UserRepository userRepository;
    private JwtService jwtService;
    private AuthenticationManager authenticationManager;

    @BeforeEach
    public void setup() {
        var passwordEncoder = new BCryptPasswordEncoder();

        userRepository = Mockito.mock(UserRepository.class);
        jwtService = Mockito.mock(JwtService.class);
        authenticationManager = Mockito.mock(AuthenticationManager.class);

        authService = new AuthService(userRepository, passwordEncoder, jwtService, authenticationManager);
    }

    @Test
    public void registerTest() {
        var jwtExampleToken = "exampleGeneratedToken";
        var req = RegisterRequestDto.builder().name("John").email("jo@mail.com").role("USER").password("Pass1").build();
        var user = User.builder().name("John").email("jo@mail.com").role(Role.USER).password("Pass1").build();

        Mockito.when(userRepository.save(user)).thenReturn(user);
        Mockito.when(jwtService.generateJwt(user)).thenReturn(jwtExampleToken);

        TokenDto token = authService.register(req);
        Assertions.assertEquals(token.getToken(), jwtExampleToken);
    }

    @Test
    public void registerExistingUser() {
        var jwtExampleToken = "exampleGeneratedToken";
        var req = RegisterRequestDto.builder().name("John").email("jo@mail.com").role("USER").password("Pass1").build();
        var user = User.builder().name("John").email("jo@mail.com").role(Role.USER).password("Pass1").build();

        var conflictException = new DataIntegrityViolationException("Some message");

        Mockito.when(userRepository.save(user)).thenThrow(conflictException);
        Mockito.when(jwtService.generateJwt(user)).thenReturn(jwtExampleToken);

        var registerException = Assertions.assertThrows(ApiException.class, () -> authService.register(req));
        Assertions.assertEquals(registerException.getMessage(), "User with this email already exists");
        Assertions.assertEquals(registerException.getHttpStatus(), HttpStatus.CONFLICT);
    }

    @Test
    public void authenticateTest() {
        var jwtExampleToken = "exampleGeneratedToken";
        var req = LoginRequestDto.builder().email("jo@mail.com").password("Pass1").build();
        var user = User.builder().name("John").email("jo@mail.com").role(Role.USER).password("Pass1").build();

        Mockito.when(userRepository.findByEmail(user.getEmail())).thenReturn(Optional.of(user));
        Mockito.when(jwtService.generateJwt(user)).thenReturn(jwtExampleToken);

        TokenDto token = authService.authenticate(req);
        Assertions.assertEquals(token.getToken(), jwtExampleToken);
    }

    @Test
    public void authenticateNonExistingUserTest() {
        var req = LoginRequestDto.builder().email("jo@mail.com").password("Pass1").build();

        Mockito.when(userRepository.findByEmail(req.getEmail())).thenReturn(Optional.empty());

        var authException = Assertions.assertThrows(ApiException.class, () -> authService.authenticate(req));
        Assertions.assertEquals(authException.getMessage(), "Invalid email or password");
        Assertions.assertEquals(authException.getHttpStatus(), HttpStatus.UNAUTHORIZED);
    }

    @Test
    public void authenticateWrongPassword() {
        var jwtExampleToken = "exampleGeneratedToken";
        var req = LoginRequestDto.builder().email("jo@mail.com").password("Pass2").build();
        var user = User.builder().name("John").email("jo@mail.com").role(Role.USER).password("Pass1").build();
        var authToken = new UsernamePasswordAuthenticationToken(req.getEmail(), req.getPassword());
        var onAuthException = new RuntimeException("Some error message");

        Mockito.when(userRepository.findByEmail(req.getEmail())).thenReturn(Optional.empty());
        Mockito.when(authenticationManager.authenticate(authToken)).thenThrow(onAuthException);
        Mockito.when(userRepository.findByEmail(user.getEmail())).thenReturn(Optional.of(user));
        Mockito.when(jwtService.generateJwt(user)).thenReturn(jwtExampleToken);

        var authException = Assertions.assertThrows(ApiException.class, () -> authService.authenticate(req));
        Assertions.assertEquals(authException.getMessage(), "Invalid email or password");
        Assertions.assertEquals(authException.getHttpStatus(), HttpStatus.UNAUTHORIZED);
    }

}
